@@ -29,16 +29,16 @@
 
 """The Python implementation of the gRPC route guide client."""
 
+from __future__ import print_function
+
 import random
 import time
 
-from grpc.beta import implementations
+import grpc
 
 import route_guide_pb2
 import route_guide_resources
 
-_TIMEOUT_SECONDS = 30
-
 
 def make_route_note(message, latitude, longitude):
   return route_guide_pb2.RouteNote(
@@ -47,15 +47,15 @@ def make_route_note(message, latitude, longitude):
 
 
 def guide_get_one_feature(stub, point):
-  feature = stub.GetFeature(point, _TIMEOUT_SECONDS)
+  feature = stub.GetFeature(point)
   if not feature.location:
-    print "Server returned incomplete feature"
+    print("Server returned incomplete feature")
     return
 
   if feature.name:
-    print "Feature called %s at %s" % (feature.name, feature.location)
+    print("Feature called %s at %s" % (feature.name, feature.location))
   else:
-    print "Found no feature at %s" % feature.location
+    print("Found no feature at %s" % feature.location)
 
 
 def guide_get_feature(stub):
@@ -64,23 +64,21 @@ def guide_get_feature(stub):
 
 
 def guide_list_features(stub):
-  rect = route_guide_pb2.Rectangle(
-      lo=route_guide_pb2.Point(
-          latitude=400000000, longitude = -750000000),
-      hi=route_guide_pb2.Point(
-          latitude = 420000000, longitude = -730000000))
-  print "Looking for features between 40, -75 and 42, -73"
+  rectangle = route_guide_pb2.Rectangle(
+      lo=route_guide_pb2.Point(latitude=400000000, longitude=-750000000),
+      hi=route_guide_pb2.Point(latitude=420000000, longitude=-730000000))
+  print("Looking for features between 40, -75 and 42, -73")
 
-  features = stub.ListFeatures(rect, _TIMEOUT_SECONDS)
+  features = stub.ListFeatures(rectangle)
 
   for feature in features:
-    print "Feature called %s at %s" % (feature.name, feature.location)
+    print("Feature called %s at %s" % (feature.name, feature.location))
 
 
 def generate_route(feature_list):
   for _ in range(0, 10):
     random_feature = feature_list[random.randint(0, len(feature_list) - 1)]
-    print "Visiting point %s" % random_feature.location
+    print("Visiting point %s" % random_feature.location)
     yield random_feature.location
     time.sleep(random.uniform(0.5, 1.5))
 
@@ -88,12 +86,12 @@ def generate_route(feature_list):
 def guide_record_route(stub):
   feature_list = route_guide_resources.read_route_guide_database()
 
-  route_iter = generate_route(feature_list)
-  route_summary = stub.RecordRoute(route_iter, _TIMEOUT_SECONDS)
-  print "Finished trip with %s points " % route_summary.point_count
-  print "Passed %s features " % route_summary.feature_count
-  print "Travelled %s meters " % route_summary.distance
-  print "It took %s seconds " % route_summary.elapsed_time
+  route_iterator = generate_route(feature_list)
+  route_summary = stub.RecordRoute(route_iterator)
+  print("Finished trip with %s points " % route_summary.point_count)
+  print("Passed %s features " % route_summary.feature_count)
+  print("Travelled %s meters " % route_summary.distance)
+  print("It took %s seconds " % route_summary.elapsed_time)
 
 
 def generate_messages():
@@ -105,27 +103,27 @@ def generate_messages():
       make_route_note("Fifth message", 1, 0),
   ]
   for msg in messages:
-    print "Sending %s at %s" % (msg.message, msg.location)
+    print("Sending %s at %s" % (msg.message, msg.location))
     yield msg
     time.sleep(random.uniform(0.5, 1.0))
 
 
 def guide_route_chat(stub):
-  responses = stub.RouteChat(generate_messages(), _TIMEOUT_SECONDS)
+  responses = stub.RouteChat(generate_messages())
   for response in responses:
-    print "Received message %s at %s" % (response.message, response.location)
+    print("Received message %s at %s" % (response.message, response.location))
 
 
 def run():
-  channel = implementations.insecure_channel('localhost', 50051)
-  stub = route_guide_pb2.beta_create_RouteGuide_stub(channel)
-  print "-------------- GetFeature --------------"
+  channel = grpc.insecure_channel('localhost:50051')
+  stub = route_guide_pb2.RouteGuideStub(channel)
+  print("-------------- GetFeature --------------")
   guide_get_feature(stub)
-  print "-------------- ListFeatures --------------"
+  print("-------------- ListFeatures --------------")
   guide_list_features(stub)
-  print "-------------- RecordRoute --------------"
+  print("-------------- RecordRoute --------------")
   guide_record_route(stub)
-  print "-------------- RouteChat --------------"
+  print("-------------- RouteChat --------------")
   guide_route_chat(stub)
 
 
